@@ -4,13 +4,17 @@ from .batches import (
     get_batches,
     update_batch_values,
     update_batch_feature_values,
+    update_vector_batch_values,
 )
 from .data import (
     get_data_list,
     get_time_values,
+    get_vector_time_values,
     get_batch_values,
+    get_vector_batch_values,
     get_multiple_data_values,
     insert_time_values,
+    insert_vector_time_values,
 )
 from .events import get_event_types, get_events
 from .files import get_file_uploads, read_file_from_file_upload
@@ -24,9 +28,13 @@ __all__ = [
     "update_batch_feature_values",
     "get_data_list",
     "get_time_values",
+    "get_vector_time_values",
     "get_batch_values",
+    "get_vector_batch_values",
     "get_multiple_data_values",
     "insert_time_values",
+    "insert_vector_time_values",
+    "update_vector_batch_values",
     "get_event_types",
     "get_events",
     "get_file_uploads",
