@@ -60,9 +60,15 @@ def get_file_uploads(
         page_size = 500
 
     def get_page(page_num: int):
-        page_response = endpoints.files.get_file_uploads(start_date=start_date, end_date=end_date, filename=filename,
-                                                         statuses=statuses, page=page_num, page_size=page_size,
-                                                         api_credentials=api_credentials)
+        page_response = endpoints.files.get_file_uploads(
+            start_date=start_date,
+            end_date=end_date,
+            filename=filename,
+            statuses=statuses,
+            page=page_num,
+            page_size=page_size,
+            api_credentials=api_credentials,
+        )
         return page_response
 
     def parse_page(page_response: dict):
@@ -130,7 +136,9 @@ def read_file_from_file_upload(
         parser_kwargs = {}
 
     # Query endpoint
-    response = endpoints.files.get_file_from_file_upload(file_upload_id=file_upload_id, api_credentials=api_credentials)
+    response = endpoints.files.get_file_from_file_upload(
+        file_upload_id=file_upload_id, api_credentials=api_credentials
+    )
 
     # Output
-    return parser(io.BytesIO(response), **parser_kwargs)
+    return parser(response, **parser_kwargs)
