@@ -1,8 +1,15 @@
-from typing import Union, List, Optional
+from typing import Union, List, Optional, Dict, Any
 from datetime import datetime, timedelta
 
 import pandas as pd
-from pydantic import BaseModel, validator, StrictStr, StrictBool, StrictInt, StrictFloat
+from pydantic import (
+    BaseModel,
+    validator,
+    StrictStr,
+    StrictBool,
+    StrictInt,
+    StrictFloat,
+)
 
 from .. import api
 
@@ -10,9 +17,67 @@ __all__ = [
     "ModelExecution",
     "get_default_model_execution",
     "set_default_model_execution",
+    "CustomModelOutput",
+    "ExecutionReport",
+    "get_default_execution_report",
+    "set_default_execution_report",
 ]
 
 
+class CustomModelOutput(BaseModel):
+    type: str
+    content: Union[StrictStr, Dict]
+
+
+class ExecutionReport(BaseModel):
+    testMode: str
+    status: str = "FAILURE"
+    customOutput: Optional[CustomModelOutput]
+    numberOfTimeValuesSuccessfullyParsed: StrictInt = 0
+    numberOfTimeValuesDeletionSuccessfullyParsed: StrictInt = 0
+    numberOfBatchValuesSuccessfullyParsed: StrictInt = 0
+    numberOfBatchCreatedOrUpdated: StrictInt = 0
+    numberOfEventCreatedOrUpdated: StrictInt = 0
+    numberOfEventValuesSuccessfullyParsed: StrictInt = 0
+    numberOfEventTagsSuccessfullyParsed: StrictInt = 0
+    numberOfBatchRelationsSuccessfullyParsed: StrictInt = 0
+    numberOfParsingErrors: StrictInt = 0
+    numberOfInvalidValues: StrictInt = 0
+    numberOfTimeVectorValuesSuccessfullyParsed: StrictInt = 0
+    numberOfBatchVectorValuesSuccessfullyParsed: StrictInt = 0
+
+    def update(
+        self,
+        time_values_parsed: StrictInt = 0,
+        time_values_deleted: StrictInt = 0,
+        batch_values_parsed: StrictInt = 0,
+        batch_created_updated: StrictInt = 0,
+        events_created_updated: StrictInt = 0,
+        event_values_parsed: StrictInt = 0,
+        event_tags_parsed: StrictInt = 0,
+        batch_relations_parsed: StrictInt = 0,
+        parsing_errors: StrictInt = 0,
+        invalid_values: StrictInt = 0,
+        time_vector_values_parsed: StrictInt = 0,
+        batch_vector_values_parsed: StrictInt = 0,
+    ):
+        self.numberOfTimeValuesSuccessfullyParsed += time_values_parsed
+        self.numberOfTimeValuesDeletionSuccessfullyParsed += time_values_deleted
+        self.numberOfBatchValuesSuccessfullyParsed += batch_values_parsed
+        self.numberOfBatchCreatedOrUpdated += batch_created_updated
+        self.numberOfEventCreatedOrUpdated += events_created_updated
+        self.numberOfEventValuesSuccessfullyParsed += event_values_parsed
+        self.numberOfEventTagsSuccessfullyParsed += event_tags_parsed
+        self.numberOfBatchRelationsSuccessfullyParsed += batch_relations_parsed
+        self.numberOfParsingErrors += parsing_errors
+        self.numberOfInvalidValues += invalid_values
+        self.numberOfTimeVectorValuesSuccessfullyParsed += time_vector_values_parsed
+        self.numberOfBatchVectorValuesSuccessfullyParsed += batch_vector_values_parsed
+
+    def set_as_default_report(self):
+        set_default_execution_report(self)
+
+
 # Credentials
 class BasicAuthCredentials(BaseModel):
     baseUrl: StrictStr
@@ -36,7 +101,7 @@ class TokenCredentials(BaseModel):
 Credentials = Union[BasicAuthCredentials, TokenCredentials]
 
 
-# Triggers
+# Cron Trigger
 class CronTrigger(BaseModel):
     type: StrictStr
     cron: StrictStr
@@ -48,7 +113,18 @@ class CronTrigger(BaseModel):
         return value
 
 
-Trigger = Union[CronTrigger]
+# File Reception Trigger
+class FileReceptionTrigger(BaseModel):
+    type: StrictStr
+
+    @validator("type")
+    def check_type(cls, value):
+        if value != "file-reception":
+            raise ValueError("Trigger type should be 'file-reception'")
+        return value
+
+
+Trigger = Union[CronTrigger, FileReceptionTrigger]
 
 
 # Model inputs & outputs
@@ -390,6 +466,16 @@ class DataValue(BaseModel):
     aggregationFunction: Optional[StrictStr]
 
 
+class VectorDataValue(BaseModel):
+    type: StrictStr
+    id: StrictStr
+    reference: StrictStr
+    dataType: StrictStr
+    valueUnit: DataUnit
+    indexUnit: DataUnit
+    aggregationFunction: Optional[StrictStr]
+
+
 class DataParameter(BaseModel):
     type: StrictStr
     sourceCodeName: StrictStr
@@ -416,6 +502,32 @@ class DataParameter(BaseModel):
             return self.value.id
 
 
+class VectorDataParameter(BaseModel):
+    type: StrictStr
+    sourceCodeName: StrictStr
+    value: Optional[VectorDataValue]
+
+    @validator("type")
+    def validate_type(cls, value):
+        if value != "VECTOR_DATA":
+            raise ValueError("'type' should be 'VECTOR_DATA'")
+        return value
+
+    @property
+    def readable_value(self):
+        if self.value is None:
+            return None
+        else:
+            return self.value.reference
+
+    @property
+    def technical_value(self):
+        if self.value is None:
+            return None
+        else:
+            return self.value.id
+
+
 class StoredContinuousDataParameter(BaseModel):
     type: StrictStr
     sourceCodeName: StrictStr
@@ -457,6 +569,19 @@ class BatchDataValue(BaseModel):
     batchType: BatchType
 
 
+class BatchVectorDataValue(BaseModel):
+    type: StrictStr
+    id: StrictStr
+    reference: StrictStr
+    dataType: StrictStr
+    valueUnit: DataUnit
+    indexUnit: DataUnit
+    aggregationFunction: Optional[StrictStr]
+    indexAggregationFunction: Optional[StrictStr]
+    batchType: BatchType
+    batchesIds: Optional[List[StrictStr]] = None
+
+
 class StoredBatchDataParameter(BaseModel):
     type: StrictStr
     sourceCodeName: StrictStr
@@ -483,6 +608,58 @@ class StoredBatchDataParameter(BaseModel):
             return self.value.id
 
 
+class StoredBatchVectorDataParameter(BaseModel):
+    type: StrictStr
+    sourceCodeName: StrictStr
+    value: Optional[BatchVectorDataValue]
+
+    @validator("type")
+    def validate_type(cls, value):
+        if value != "STORED_BATCH_VECTOR_DATA":
+            raise ValueError("'type' should be 'STORED_BATCH_VECTOR_DATA'")
+        return value
+
+    @property
+    def readable_value(self):
+        if self.value is None:
+            return None
+        else:
+            return self.value.reference
+
+    @property
+    def technical_value(self):
+        if self.value is None:
+            return None
+        else:
+            return self.value.id
+
+
+class StoredTimeVectorDataParameter(BaseModel):
+    type: StrictStr
+    sourceCodeName: StrictStr
+    value: Optional[VectorDataValue]
+
+    @validator("type")
+    def validate_type(cls, value):
+        if value != "STORED_TIME_VECTOR_DATA":
+            raise ValueError("'type' should be 'STORED_TIME_VECTOR_DATA'")
+        return value
+
+    @property
+    def readable_value(self):
+        if self.value is None:
+            return None
+        else:
+            return self.value.reference
+
+    @property
+    def technical_value(self):
+        if self.value is None:
+            return None
+        else:
+            return self.value.id
+
+
 class ComputedContinuousDataParameter(BaseModel):
     type: StrictStr
     sourceCodeName: StrictStr
@@ -812,6 +989,8 @@ InstanceParameter = Union[
     DataParameter,
     StoredContinuousDataParameter,
     StoredBatchDataParameter,
+    StoredTimeVectorDataParameter,
+    StoredBatchVectorDataParameter,
     ComputedContinuousDataParameter,
     ComputedBatchDataParameter,
     BatchTimeDataParameter,
@@ -863,6 +1042,16 @@ SingleObservationContext = Union[
 ]
 
 
+class ExecutionContext(BaseModel):
+    type: StrictStr
+    initiator: StrictStr
+    lastSuccessfulExecutionInstant: Optional[datetime]
+    currentExecutionInstant: datetime
+    customData: Optional[Any]
+    uploadEventId: Optional[StrictStr]
+    fileName: Optional[StrictStr]
+
+
 class ModelInstance(BaseModel):
     id: StrictStr
     trigger: Trigger
@@ -877,8 +1066,9 @@ class ModelInstance(BaseModel):
 class ModelExecution(BaseModel):
     testMode: StrictBool
     credentials: Optional[Credentials]
-    lastSuccessfulExecutionInstant: datetime
+    lastSuccessfulExecutionInstant: Optional[datetime]
     currentExecutionInstant: datetime
+    executionContext: ExecutionContext
     pythonModelInstance: ModelInstance
 
     def set_as_default_model_execution(self):
@@ -973,11 +1163,33 @@ class ModelExecution(BaseModel):
     def get_data_input_dict(
         self,
         data_type: str = "any",
+        values_type: str = "any",
         strict: bool = False,
         mode: str = "object",
         exclude_empty_values: bool = True,
     ):
-        if data_type == "any":
+        """
+        Parameters
+        ----------
+        data_type: {'any, 'time', 'batch'}
+        values_type: {'any', 'scalar', 'vector'}
+        strict: bool
+        mode: {'object', 'reference', 'id'}
+        exclude_empty_values: bool
+        """
+        if data_type == "any" and values_type == "any":
+            input_types = [
+                "DATA",
+                "STORED_CONTINUOUS_DATA",
+                "STORED_BATCH_DATA",
+                "COMPUTED_CONTINUOUS_DATA",
+                "COMPUTED_BATCH_DATA",
+                "BATCH_TIME_DATA",
+                "BATCH_TAG_KEY",
+                "STORED_TIME_VECTOR_DATA",
+                "STORED_BATCH_VECTOR_DATA",
+            ]
+        elif data_type == "any" and values_type == "scalar":
             input_types = [
                 "DATA",
                 "STORED_CONTINUOUS_DATA",
@@ -987,11 +1199,41 @@ class ModelExecution(BaseModel):
                 "BATCH_TIME_DATA",
                 "BATCH_TAG_KEY",
             ]
-        elif data_type == "time":
-            input_types = ["STORED_CONTINUOUS_DATA", "COMPUTED_CONTINUOUS_DATA"]
+        elif data_type == "any" and values_type == "vector":
+            input_types = [
+                "STORED_TIME_VECTOR_DATA",
+                "STORED_BATCH_VECTOR_DATA",
+            ]
+        elif data_type == "time" and values_type == "any":
+            input_types = [
+                "STORED_CONTINUOUS_DATA",
+                "COMPUTED_CONTINUOUS_DATA",
+                "STORED_TIME_VECTOR_DATA",
+            ]
             if strict is False:
                 input_types.append("DATA")
-        elif data_type == "batch":
+        elif data_type == "time" and values_type == "scalar":
+            input_types = [
+                "STORED_CONTINUOUS_DATA",
+                "COMPUTED_CONTINUOUS_DATA",
+            ]
+            if strict is False:
+                input_types.append("DATA")
+        elif data_type == "time" and values_type == "vector":
+            input_types = ["STORED_TIME_VECTOR_DATA"]
+            if strict is False:
+                input_types.append("VECTOR_DATA")
+        elif data_type == "batch" and values_type == "any":
+            input_types = [
+                "STORED_BATCH_DATA",
+                "COMPUTED_BATCH_DATA",
+                "BATCH_TIME_DATA",
+                "BATCH_TAG_KEY",
+                "STORED_BATCH_VECTOR_DATA",
+            ]
+            if strict is False:
+                input_types.append("DATA")
+        elif data_type == "batch" and values_type == "scalar":
             input_types = [
                 "STORED_BATCH_DATA",
                 "COMPUTED_BATCH_DATA",
@@ -1000,6 +1242,10 @@ class ModelExecution(BaseModel):
             ]
             if strict is False:
                 input_types.append("DATA")
+        elif data_type == "batch" and values_type == "vector":
+            input_types = ["STORED_BATCH_VECTOR_DATA"]
+            if strict is False:
+                input_types.append("VECTOR_DATA")
         else:
             raise ValueError(f"Unknown 'data_type': {data_type}")
 
@@ -1012,11 +1258,33 @@ class ModelExecution(BaseModel):
     def get_data_output_dict(
         self,
         data_type: str = "any",
+        values_type: str = "any",
         strict: bool = False,
         mode: str = "object",
         exclude_empty_values: bool = True,
     ):
-        if data_type == "any":
+        """
+        Parameters
+        ----------
+        data_type: {'any, 'time', 'batch'}
+        values_type: {'any', 'scalar', 'vector'}
+        strict: bool
+        mode: {'object', 'reference', 'id'}
+        exclude_empty_values: bool
+        """
+        if data_type == "any" and values_type == "any":
+            output_types = [
+                "DATA",
+                "STORED_CONTINUOUS_DATA",
+                "STORED_BATCH_DATA",
+                "COMPUTED_CONTINUOUS_DATA",
+                "COMPUTED_BATCH_DATA",
+                "BATCH_TIME_DATA",
+                "BATCH_TAG_KEY",
+                "STORED_BATCH_VECTOR_DATA",
+                "STORED_TIME_VECTOR_DATA",
+            ]
+        elif data_type == "any" and values_type == "scalar":
             output_types = [
                 "DATA",
                 "STORED_CONTINUOUS_DATA",
@@ -1026,11 +1294,38 @@ class ModelExecution(BaseModel):
                 "BATCH_TIME_DATA",
                 "BATCH_TAG_KEY",
             ]
-        elif data_type == "time":
+        elif data_type == "any" and values_type == "vector":
+            output_types = [
+                "STORED_TIME_VECTOR_DATA",
+                "STORED_BATCH_VECTOR_DATA",
+            ]
+        elif data_type == "time" and values_type == "any":
+            output_types = [
+                "STORED_CONTINUOUS_DATA",
+                "COMPUTED_CONTINUOUS_DATA",
+                "STORED_TIME_VECTOR_DATA",
+            ]
+            if strict is False:
+                output_types.append("DATA")
+        elif data_type == "time" and values_type == "scalar":
             output_types = ["STORED_CONTINUOUS_DATA", "COMPUTED_CONTINUOUS_DATA"]
             if strict is False:
                 output_types.append("DATA")
-        elif data_type == "batch":
+        elif data_type == "time" and values_type == "vector":
+            output_types = ["STORED_TIME_VECTOR_DATA"]
+            if strict is False:
+                output_types.append("VECTOR_DATA")
+        elif data_type == "batch" and values_type == "any":
+            output_types = [
+                "STORED_BATCH_DATA",
+                "COMPUTED_BATCH_DATA",
+                "BATCH_TIME_DATA",
+                "BATCH_TAG_KEY",
+                "STORED_BATCH_VECTOR_DATA",
+            ]
+            if strict is False:
+                output_types.append("DATA")
+        elif data_type == "batch" and values_type == "scalar":
             output_types = [
                 "STORED_BATCH_DATA",
                 "COMPUTED_BATCH_DATA",
@@ -1039,6 +1334,10 @@ class ModelExecution(BaseModel):
             ]
             if strict is False:
                 output_types.append("DATA")
+        elif data_type == "batch" and values_type == "vector":
+            output_types = ["STORED_BATCH_VECTOR_DATA"]
+            if strict is False:
+                output_types.append("VECTOR_DATA")
         else:
             raise ValueError(f"Unknown 'data_type': {data_type}")
 
@@ -1061,6 +1360,7 @@ class ModelExecution(BaseModel):
             "output_parameters": self.get_output_dict(mode="object"),
             "output_parameter_references": self.get_output_dict(mode="reference"),
             "output_parameter_ids": self.get_output_dict(mode="id"),
+            "custom_input_data": self.executionContext.customData,
         }
 
     def get_execution_argument_value(self, argument_name: str):
@@ -1087,3 +1387,24 @@ def set_default_model_execution(event: ModelExecution):
 def get_default_model_execution():
     global DEFAULT_EVENT
     return DEFAULT_EVENT
+
+
+# Current summary
+DEFAULT_EXECUTION_REPORT: Optional[ExecutionReport] = None
+
+
+def set_default_execution_report(output_summary: ExecutionReport):
+    """
+    Put a summary into a global variable, for external use
+    Parameters
+    ----------
+    output_summary: OutputSummary
+    """
+
+    global DEFAULT_EXECUTION_REPORT
+    DEFAULT_EXECUTION_REPORT = output_summary
+
+
+def get_default_execution_report():
+    global DEFAULT_EXECUTION_REPORT
+    return DEFAULT_EXECUTION_REPORT
