@@ -351,7 +351,8 @@ class VectorBatchValuesOutput:
             ),
             invalid_values=summary.numberOfInvalidValues
             + response.get("numberOfValuesRejected", 0),
-            parsing_errors=summary.numberOfParsingErrors + len(response.get("errors", [])),
+            parsing_errors=summary.numberOfParsingErrors
+            + len(response.get("errors", [])),
         )
         return response
 
@@ -414,7 +415,7 @@ class CustomTextOutput:
         self.type = "text"
         self.content = content
 
-    def send_to_oianalytics(self, execution_report: Optional[ExecutionReport]):
+    def send_to_oianalytics(self, execution_report: Optional[ExecutionReport] = None):
         # Get the default execution report if not provided
         if execution_report is None:
             execution_report = get_default_execution_report()
@@ -430,7 +431,7 @@ class CustomJsonOutput:
         self.type = "json"
         self.content = content
 
-    def send_to_oianalytics(self, execution_report: Optional[ExecutionReport]):
+    def send_to_oianalytics(self, execution_report: Optional[ExecutionReport] = None):
         # Get the default execution report if not provided
         if execution_report is None:
             execution_report = get_default_execution_report()
